@@ -1,10 +1,10 @@
 /*
- * Copyright 2005-2006 Gentoo Foundation
+ * Copyright 2005-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxinc.h,v 1.6 2006/01/14 01:39:55 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxinc.h,v 1.12 2008/10/19 18:55:33 grobian Exp $
  *
- * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
  *
  * Make sure all of the common stuff is setup as we expect
  */
@@ -13,6 +13,7 @@
 #define _PAX_INC_H
 
 #include "porting.h"
+#include "xfuncs.h"
 
 #ifndef VERSION
 # define VERSION "cvs"
@@ -22,7 +23,7 @@
 #include "elf.h"
 #include "paxelf.h"
 
-/* MACH-O sucks */
+/* Mach-O love */
 #include "macho.h"
 #include "paxmacho.h"
 
@@ -33,6 +34,7 @@ typedef struct {
 	int fd;
 	const char *filename;
 	size_t skip;
+	char *extfn;
 } archive_handle;
 #else
 typedef void archive_handle;
@@ -55,7 +57,7 @@ typedef struct {
 			char mode[8];
 			char size[10];
 			char magic[2];
-		} formated;
+		} formatted;
 	} buf;
 #endif
 } archive_member;
@@ -94,10 +96,15 @@ archive_member *ar_next(archive_handle *);
 #define RED       COLOR("31", "01")
 #define YELLOW    COLOR("33", "01")
 
+/* constant pointer to a constant buffer ... each program needs to set this */
+extern const char * const argv0;
+
 /* we need the space before the last comma or we trigger a bug in gcc-2 :( */
 #define warn(fmt, args...) \
-	fprintf(stderr, "%s%s%s: " fmt "\n", RED, argv0, NORM , ## args) 
+	fprintf(stderr, "%s%s%s: " fmt "\n", RED, argv0, NORM , ## args)
 #define warnf(fmt, args...) warn("%s%s%s(): " fmt, YELLOW, __FUNCTION__, NORM , ## args)
+#define warnp(fmt, args...) warn(fmt ": %s" , ## args , strerror(errno))
+#define warnfp(fmt, args...) warnf(fmt ": %s" , ## args , strerror(errno))
 #define _err(wfunc, fmt, args...) \
 	do { \
 	wfunc(fmt, ## args); \
@@ -105,5 +112,6 @@ archive_member *ar_next(archive_handle *);
 	} while (0)
 #define err(fmt, args...) _err(warn, fmt, ## args)
 #define errf(fmt, args...) _err(warnf, fmt, ## args)
+#define errp(fmt, args...) _err(warnp, fmt , ## args)
 
 #endif /* _PAX_INC_H */
